#!/usr/bin/perl
#
#===============================================================================
#
#         FILE:  WPXMLCreator.pl
#
#        USAGE:  ./WPXMLCreator.pl [requires options] <optional options>
#
#  DESCRIPTION:
#
#      OPTIONS:
# REQUIREMENTS:  Perl v. 5.8.8 or higher
#       AUTHOR:  Dimio (dimio@dimio.org)
#         HOME:  http://dimio.org/wpxmlcreator
#      VERSION:  2.1.0
#      CREATED:  21.03.2011 20:07:47
#   $Revision$:
#         $Id$:
#       $Date$:
#===============================================================================
#
require 5.008_008;
use strict;
use warnings;
use utf8;
#use Carp;
use Getopt::Std;
our $VERSION = '2.10';
local $| = 1;    # use autoflush
###  GET OPTIONS ###
# Set default options
my %option = (
    'a' => q{},                      # authors.txt file
    'i' => q{},                      # images.txt file
    'o' => './wp-satellit_1.xml',    # output file name
    'n' => '24-24',                  # hours btw pub articles
    'l' => 1,                        # output file limit factor (Mb)
    'c' => 'open',                   # post comment status
    'p' => 'open',                   # post pingback status
    'f' => 'UTF-8',                  # input file encoding

    'MB_FACTOR'     => 1_048_576,    # количество байт в Мб
    'start_year'    => q{},
    'end_year'      => q{},
    'EOL'           => "\n",         # символ перевода строки
);

# стандартные значения для преобразования
# вывода localtime в реальную дату;
# число секунд в сутках
my %time_conv_const = (
    sec_per_day     => 86_400,
    sec_per_year    => 31_557_600,
    sec_per_hour    => 3600,
    month_delta     => 1,
    year_delta      => 1900,
    rnd_hour_delta  => 0,
);

# Get user options
getopts( 's:a:i:o:n:l:c:p:y:f:w:t:uh', \%option );
# если не задан исходный файл и вызов с ключем help - показать справку и выйти
die usage(),"\n" if ( $option{h} || !$option{s} );
check_option (\%option);


# начало нумерации частей выходных файлов
my $fpart = 1;

# массив, в котором будут лежать ссылки на картинки
my @images_list;

# массив с именами авторов
my @authors;
########## MAIN ##########
$option{o} =~ s/(\w+)([.]\w+)/$1_$fpart$2/ixms;
xml_main_build( 'start', \%option );
create_xml( \%option, \%time_conv_const );
xml_main_build( 'end', \%option );

print "\nDone!\n";
exit 0;
########## FUNCTIONS ##########
sub check_option {
    # проверка, изменение, установка опций
    my $option = shift;

 # :TODO:27.03.2011 09:33:10:: сделать проверку всех передаваемых пользователем опций

    # назначается предельный размер выходного файла в Мб
    if ($option->{l} =~ m/^[\d.]+$/x) {
        $option->{l} = $option->{MB_FACTOR} * $option->{l};
    }
    else { die 'Option -l incorrect (' .$option->{l}. '), check this manually',"\n"; }

    # если задан диапазон годов для назначения дат публикации постов -
    # разобрать его на составные
    if ($option->{y}) {
        if ($option->{y} =~ m/^(\d{4})\-(\d{4})$/msx) {
            $option->{start_year} = $1;
            $option->{end_year}   = $2;
        }
        else { die 'Option -y incorrect (' .$option->{y}. '), check this manually',"\n"; }
    }

    # если указан тип ОС как Win - установить соотв. окончания строк
    # для чтения файла со статьями
    if ($option->{w}) {
        $option->{EOL} = "\r\n";
    }

    return 1;
}

sub create_xml {
    my $option = shift;
    my $time_conv_const = shift;

    my (
        $article_header, $article_body,
        @article_tags,   @article_cats
    );

    open my $in_file, "<:encoding($option->{f})", $option->{s}
        or die "Can't open file $option->{s}: " . $!, "\n";
    print "\nParsing input file: $option->{s}\n";
    print "Process\n";
    {
        local $/ = $option->{EOL} x 2; # локальный перевод строки - два обычных (\n\n)
        #my $EOL = qr|$option->{EOL}|;

        while (<$in_file>) {
            print process();
            chomp;
            s/^\s+//xms; #добавлен флаг /s (вкл. в '.' \n); /m; /x
            s/\s+$//xms;
            $_ = dos_to_unix($_) if $option->{w};

            #print $_,"\n";exit;

            if (m/^<header>(.+)<\/header>\n
                   <body>(.+)<\/body>\n
                   <tags>(.*?)<\/tags>\n
                   <category>(.*?)<\/category>$/ixms) {
                ( $article_header, $article_body ) = ( $1, $2 );
                @article_tags = split( /,/xms, $3 );
                @article_cats = split( /,/xms, $4 );

                {
                    local $/ = "\n";
                    xml_article_build( $option, $article_header,
                        $article_body, \@article_tags, \@article_cats,
                        $time_conv_const,
                    );
                }
                $#article_tags = -1;
                $#article_cats = -1;
            }
            elsif (m/^\s*$/xms) {}
            else {
                die
                    "Input file format or encoding incorrect.\n";
            }

            if ( -s $option->{o} >= $option->{l} ) {
                print "\n$option->{o}: file creation is complete.\n";
                $fpart++;
                xml_main_build('end', $option);
                $option->{o} =~ s/(\w+)_\d+(\.\w+)/$1_$fpart$2/ixms;
                xml_main_build('start', $option);
                redo;
            }
        }
    }
    close($in_file) or die "unable to close $option->{s} : $!\n";

    return 1;
}

sub xml_main_build {
    my $position = shift;
    my $option   = shift;

    if ( $position eq 'start' ) {
        open my $output_file, '>:encoding(UTF-8)', $option->{o}
            or die "Can't open file $option->{o}: " . $!, "\n";
        print {$output_file} '<?xml version="1.0" encoding="UTF-8"?>', "\n\n";
        print {$output_file} '<!-- Wordpress XML.'
            . ' Generated WPXMLCreator v. '
            . $VERSION . ' -->
<!-- Copyright 2009 Dimio <dimio@dimio.org> -->
<!-- Homepage: http://www.dimio.org/wpxmlcreator -->', "\n";
        print $output_file <<'END_RSS_COMMENT';
<rss version="2.0"
xmlns:excerpt="http://wordpress.org/export/1.0/excerpt/"
xmlns:content="http://purl.org/rss/1.0/modules/content/"
xmlns:wfw="http://wellformedweb.org/CommentAPI/"
xmlns:dc="http://purl.org/dc/elements/1.1/"
xmlns:wp="http://wordpress.org/export/1.0/"
>
END_RSS_COMMENT
        print $output_file '<channel>', "\n";
        close($output_file) or die "unable to close $option->{o} : $!\n";
    }

    if ( $position eq 'end' ) {
        open( my $output_file, '>>:encoding(UTF-8)', $option->{o} )
            or die "Can't open file $option->{o}: " . $!, "\n";
        print $output_file '
</channel>
</rss>', "\n";
        close($output_file) or die "unable to close $option->{o} : $!\n";
    }

    return 1;
}

sub xml_article_build {
    my ( $option, $article_header, $article_body, $article_tags_ref,
        $article_cats_ref, $time_conv_const,
    ) = @_;

 # :TODO:16.02.2012 20:30:42:dimio.org: Вынести в ф-ю проверки опций?
    #получаем границы интервала между публикацией статей
    my ($minH, $maxH) = split(/-/,$option->{n},2)
        or die "Bad time interval format: $option->{n}\n";

    open( my $output_file, '>>:encoding(UTF-8)', $option->{o} )
        or die "Can't open file $option->{o}: " . $!, "\n";

    print $output_file "\n<item>\n";
    print $output_file "<title>" . $article_header . "</title>" . "\n";
    print $output_file '<dc:creator><![CDATA['
        . get_rand_author($option)
        . ']]></dc:creator>' . "\n";

    foreach my $category ( @{$article_cats_ref} ) {
        shift;
        chomp($category);
        print $output_file '<category><![CDATA['
            . $category
            . ']]></category>'
            . '<category domain="category" nicename="'
            . translate($category)
            . '"><![CDATA['
            . $category
            . ']]></category>',
            "\n";
    }
    foreach my $tag ( @{$article_tags_ref} ) {
        shift;
        chomp($tag);
        print $output_file '<category domain="tag"><![CDATA[' . $tag
            . ']]></category><category domain="tag" nicename="'
            . translate($tag) . '"><![CDATA[' . $tag . ']]></category>', "\n";
    }

    print $output_file '<content:encoded><![CDATA['
        . get_rand_image($option)
        . $article_body
        . ']]></content:encoded>' . "\n";
    print $output_file '<wp:post_date>'
        . set_date_time($option, $time_conv_const, $minH, $maxH)
        . '</wp:post_date>' . "\n";
    print $output_file '<wp:comment_status>'
        . $option->{c}
        . '</wp:comment_status>' . "\n";
    print $output_file '<wp:ping_status>'
        . $option->{p}
        . '</wp:ping_status>' . "\n";
    print $output_file '<wp:post_name>'
        . translate($article_header)
        . '</wp:post_name>' . "\n";
    print $output_file '<wp:status>publish</wp:status>
<wp:post_type>post</wp:post_type>
</item>' . "\n";

    close($output_file) or die "unable to close $option->{o} : $!\n";

    return 1;
}

sub dos_to_unix {
    my $line = shift;

    $line =~ s{\r\n}{\n}g;

    return $line;
}

sub translate {
# Слегка изменённая под свои нужды функция транслитерации
# от Александра (http://eax.me, afiskon@gmail.com)
    my $line = shift;

    $line = "\L$line";

    $line =~ tr{абвгдезийклмнопрстуфхцыэ}{abvgdezijklmnoprstufxcye};
    $line =~ s/ё/yo/g;
    $line =~ s/ж/zh/g;
    $line =~ s/ч/ch/g;
    $line =~ s/ш/sh/g;
    $line =~ s/щ/shh/g;
    $line =~ s/ю/yu/g;
    $line =~ s/я/ya/g;
    $line =~ s/\s/-/g;
    $line =~ s/[^\w-]//g;

    return $line;
}

sub set_date_time {
    my ($option, $time_conv_const, $minH, $maxH) = @_;

    # если при запуске скрипта задан диапазон годов,
    # внутри которого надо генерировать даты -
    # установить year_delta для localtime в соотв. значение
    if ($option->{y}) {
        set_time_interval($option, $time_conv_const);
    }


    # не нужен, т.к. интервал между публикациями задается в часах
    #my $day_delta = $day_num
        #* $time_conv_const{sec_per_day}; # сдвиг от текущей даты, в секундах


    my ( $HOUR, $DAY, $MONTH, $YEAR ) =
        ( localtime( time + $time_conv_const->{rnd_hour_delta} ) )[2 .. 5];

    my $DATE = join( q{-},
        $YEAR + $time_conv_const{year_delta},
        $MONTH + $time_conv_const{month_delta},
        $DAY
    );

    $DATE .= q{ } . join( q{:},  $HOUR, int( rand(59) ), int( rand(59) ) );

    $time_conv_const->{rnd_hour_delta} +=
        (int(rand( ($maxH+1)-$minH ))+$minH) * $time_conv_const{sec_per_hour};

    #print $DATE,"\n";
    return $DATE;
}

sub set_time_interval {
        my ($option, $time_conv_const) = @_;

        my $CURRENT_YEAR           = (localtime)[5] + $time_conv_const->{year_delta};
        my $CURRENT_YEAR_DELTA     = (localtime)[5];
        my $START_YEAR_DELTA       = $CURRENT_YEAR - $option->{start_year} +1;
        my $END_YEAR_DELTA         = $CURRENT_YEAR - $option->{end_year} +1;

        while (1) {
            $time_conv_const->{year_delta} =
                1900 - ( int(rand($END_YEAR_DELTA)) + int(rand($START_YEAR_DELTA)) );

            my $tmp = $CURRENT_YEAR_DELTA + $time_conv_const->{year_delta};

            if ( $option->{start_year} <= $tmp && $tmp <= $option->{end_year }) { return 1; }
        }
}

sub process {

    #local $| = 1;
    return q{.};
}

sub get_rand_author {

# Функция создаёт массив с именами авторов постов если указан файл с авторами
# и возвращает случайное имя автора или имя 'admin', если файл не указан
    my $option = shift;

    return 'admin' if !$option->{a};

# если массив имён авторов пустой - заполнить его именами из файла
    if ( !@authors ) {
        my $authors_fh;
        open( $authors_fh, "<:encoding($option->{f})", $option->{a} ) or die "unable to open $option->{a} : $!\n";
            @authors = <$authors_fh>;
        close($authors_fh) or die "unable to close $option->{a} : $!\n";
    }

    my $author = @authors[ int( rand( scalar @authors ) ) ];
    chomp $author;

    return $author;
}

sub get_rand_image {

# Функция создаёт массив ссылок на картинки если указан файл с такими ссылками
# (или возвращает пустую строку, если не указан)
# и возвращает случайную ссылку на картинку, окружённую html-кодом
    my ( $option, $images_list ) = shift;

# если не задан файл со ссылками на картинки - выходим из функции
# с возвратом пустой строки
    return q{} if !$option->{i};

# если массив с картинками пустой - заполнить его ссылками из файла
    if ( !@images_list ) {
        my $images_fh;
        open( $images_fh, "<:encoding($option->{f})", $option->{i} )
            or die "unable to open $option->{i} : $!\n";
        @images_list = <$images_fh>;
        close($images_fh) or die "unable to close $option->{i} : $!\n";
    }

# когда массив уже существует и заполнен ссылками -
# возьмем случайную ссылку на картинку, вставим ее в html-код и вернём для добавления к посту
# v.2.0.3 - картинки лежат в файле уже с нужной html-разметкой, генератор ничего не добавляет
    my $image = @images_list[ int( rand( scalar @images_list ) ) ];
    chomp $image;
    #$image = '<a href="' . $image . '"><img src="' . $image . '" /></a>';

    return $image;
}

sub usage {
    return <<"END_USAGE";
WPXMLCreator v. $VERSION by Dimio (dimio\@dimio.org)

Usage: $0 [requires options] <optional options>

Requires options:
    -s : input file name        - path and name input file

Optional options
    -a : authors file name      - path and name authors file, default is empty
    -i : images file name       - path and name images file, default is empty
    -o : output file name       - path and name output file, default = 'wp-sattellit.xml'
    -n : articles per day       - time interval (in hours) between articles publications, default= 24-24
    -l : otput file size limit  - maximum file size in megabytes, default = 1 Mb
    -c : comment status         - 'open' or 'close', default = open
    -p : ping status            - 'open' or 'close', default = open
    -y : years interval         - random year will be taken from the specified range of years
    -f : input encoding         - set encoding for all input files (articles, authors, images), default is UTF-8
    -w : input file format      - if specifies - set articles file format as a 'DOS'

    -h : help                   - show this help and exit

Example:
$0 -s articles_example.txt -a authors.txt -i images.txt -o satellit.xml -n 5-18 -l 1.5 -c open -p close -f CP1251 -y 2007-2009

See more on http://www.dimio.org/wpxmlcreator
END_USAGE
}
